/* AbulEduCameraWorkerV1
 *
 * @warning aucun traitement d'erreur n'est pour l'instant implémenté
 * @see https://redmine.ryxeo.com/projects/leterrier-developpeu/repository
 * @author 2015 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
 * @see The GNU Public License (GPL)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef ABULEDUCAMERAWORKER_H
#define ABULEDUCAMERAWORKER_H

#include <QObject>
//#include <QSize>
#include <QImage>
#include <QTimer>
#include <QTime>
#include <QDebug>

// opencv
#ifdef WITH_OPENCV2
  #include <opencv2/core/core.hpp>
  #include <opencv2/highgui/highgui.hpp>
  #include <opencv2/imgproc/imgproc.hpp>
#endif

//#include "config.h"

///
/// \brief The AbulEduCameraWorker manages webcams
///
/// Cette classe gère l'acquisition d'images par le biais d'une webcam connectée au pc
///
/// Elle est prévue pour fonctionner dans un thread indépendant du thread principal.
/// Les échanges avec cette classe se font par le biais des signaux et slots qui sont thread-safe
///

//Q_DECLARE_METATYPE(cv::Mat)

class AbulEduCameraWorker : public QObject
{
    Q_OBJECT
public:
    explicit AbulEduCameraWorker(QObject *parent = 0);
    ~AbulEduCameraWorker();

private:
    /* capture une frame */
    void getFrame();



    bool m_localDebug;
    bool m_isCameraConnected;
    bool m_inited;
    QImage *m_image;
    int m_deviceNumber;
    int m_resWidth;
    int m_resHeight;
    cv::VideoCapture m_camera;
    cv::Mat m_frame;
    cv::Rect m_rectROI;
    QTime m_time;

signals:
    ///
    /// \brief signalAbeCamWorkerIsCameraConnected passe true si la caméra est connectée
    ///
    void signalAbeCamWorkerIsCameraConnected(bool);

    ///
    /// \brief signalAbeCamWorkerImageCvMatReady passe la cv::Mat de l'image capturée
    ///
    void signalAbeCamWorkerImageCvMatReady(cv::Mat);

    void signalAbeCamWorkerCroppedImageCvMatReady(cv::Mat);

    ///
    /// \brief signalAbeCamWorkerQImageReady passe la QImage de l'image capturée
    ///
    void signalAbeCamWorkerQImageReady(QImage);

    void signalAbeCamWorkerCroppedQImageReady(QImage);

    ///
    /// \brief signalAbeCamWorkerImageSize passe les dimensions de l'image capturée
    ///
    void signalAbeCamWorkerImageSize(int, int);

    ///
    /// \brief signalAbeCamWorkerError retourne un numéro d'erreur
    ///
    ///  0 = Tout va bien, 1 = Camera non connectée
    ///
    void signalAbeCamWorkerError(int);

public slots:
    ///
    /// \brief slotAbeCamWorkerSetDevice initialise la camera passée en paramètre
    ///
    /// \note Par défaut, la caméra passée en paramètre est initialisée dans sa plus grande résolution.
    /// \param deviceNumber
    ///
    void slotAbeCamWorkerSetDevice(int deviceNumber, int width, int height);

    ///
    /// \brief slotAbeCamWorkerSetResolution passe la caméra dans la résolution demandée
    /// \param resWidth Nombre de pixels en largeur
    /// \param resHeight Nombre de pixels en hauteur
    ///
    void slotAbeCamWorkerSetResolution(int resWidth, int resHeight);

    ///
    /// \brief slotAbeCamWorkerGetFullImage capture une image sous forme de QImage
    ///
    /// \note le retour se fait par le signal signalAbeCamWorkerQImageReady(QImage)
    ///
    void slotAbeCamWorkerGetFullImage();

    void slotAbeCamWorkerGetCroppedImage();

    ///
    /// \brief slotAbeCamWorkerGetFullCvMatImage capture une image sous forme de cv::Mat (opencv2)
    ///
    /// \note le retour se fait par le signal signalAbeCamWorkerImageCvMatReady(cv::Mat)
    ///
    void slotAbeCamWorkerGetFullCvMatImage();

    void slotAbeCamWorkerGetCroppedCvMatImage();

    ///
    /// \brief slotAbeCamWorkerSetROI fixe la Region Of Interest de l'image (rectangle)
    /// \param posX
    /// \param posY
    /// \param width
    /// \param height
    ///
    void slotAbeCamWorkerSetROI(int posX, int posY, int width, int height);
private slots:
    void initCapture();

};

#endif // ABULEDUCAMERAWORKER_H

