#ifndef ABULEDUQRCODEREADERV1_H
#define ABULEDUQRCODEREADERV1_H

#include <QObject>
#include <QDebug>
#include <QStringList>
#include <QImage>

// Pour décoder les codeBar et QRCodes
#include <zbar.h>


// opencv
#ifdef WITH_OPENCV2
  #include <opencv2/core/core.hpp>
  #include <opencv2/highgui/highgui.hpp>
  #include <opencv2/imgproc/imgproc.hpp>
#endif


struct codeBar{
    QString type;
    QString data;
    QVector<QPair<int, int> > points;
};

class AbulEduQrCodeReaderV1 : public QObject
{
    Q_OBJECT
public:
    explicit AbulEduQrCodeReaderV1(QObject *parent = 0);
    ~AbulEduQrCodeReaderV1();

private:
    bool m_localDebug;
    zbar::ImageScanner *m_scanner;
    bool m_scannerInited;
signals:
    ///
    /// \brief signalAbeQrCodeReaderDecodedCvMatImageReady
    ///
    /// Une image décodée est prête, elle est passée sous forme de cv::Mat)
    ///
    void signalAbeQrCodeReaderDecodedCvMatImageReady(cv::Mat);

    ///
    /// \brief signalAbeQrCodeReaderNewDecodedDataList des données ont été décodées.
    ///
    /// Les données sont passées dans le signal dans une struct codeBar
    ///
    void signalAbeQrCodeReaderNewDecodedDataList(codeBar);
public slots:

    ///
    /// \brief slotAbeQrCodeReaderSetCvMatImage passe une image au scanner de codeBar
    /// \param frame L'image a décoder sous forme de cv::Mat
    /// \param noframe Si true, on ne renvoie pas l'image, si false on renvoie l'image avec les zones décodées en bleu
    ///
    void slotAbeQrCodeReaderSetCvMatImage(cv::Mat frame, bool noframe = false);

    ///
    /// \brief abeQrCodeReaderInit initialise le scanner.
    ///
    /// Cette fonction doit être appelée avant tout autre accès à cet objet.
    ///
    void abeQrCodeReaderInit();

};

#endif // ABULEDUQRCODEREADERV1_H
