#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_cameraController(0),
    m_QRCodesReader(0),
    m_scannerThread(0)

{
    ui->setupUi(this);
    lbFPS = new QLabel("0 fps");
    lbRes = new QLabel("0x0 px");
    ui->statusBar->addWidget(lbFPS);
    ui->statusBar->addWidget(lbRes);
}

MainWindow::~MainWindow()
{
    delete ui;
    if(m_scannerThread)
    {
        m_scannerThread->quit();
        m_scannerThread->wait();
    }
}

void MainWindow::on_btnStart_clicked()
{
    /* On crée un camera controleur s'il n'y en a pas déjà un */
    if(!m_cameraController)
    {
        m_cameraController = new AbulEduCameraControllerV1(this);
        /* Pour la démo, on utilise la première caméra trouvée par le système (sous linux /dev/video0)
         * On utilise une résolution assez faible pour ne pas ralentir le systeme
         * et on fixe 10 images par seconde pour la même raison
         * Mais la caméra peut ne pas honorer cette demande et retourner une image plus petite
         * Ou une fréquence inférieure
         */

        m_cameraController->abeCameraInitCamera(0, 1024, 728);
        m_cameraController->abeCameraSetFrameRefreshInterval(DEFAULT_SCAN_REFRESH_INTERVAL);
        connect(m_cameraController, SIGNAL(signalCameraFPSAverage(int)),
                this,               SLOT(updateFPS(int)), Qt::UniqueConnection);
        connect(m_cameraController, SIGNAL(signalAbeCameraImageSize(int,int)),
                this, SLOT(updateResolution(int, int)), Qt::UniqueConnection);
    }
}

void MainWindow::updateFPS(int fps)
{
    lbFPS->setText(QString::number(fps)+" fps");
}

void MainWindow::updateResolution(int width, int height)
{
    lbRes->setText(QString::number(width)+"x"+QString::number(height)+" px");
}

void MainWindow::on_btnReadQRCodes_clicked()
{
    if(!m_QRCodesReader)
    {
        m_QRCodesReader = new AbulEduQrCodeReaderV1();
        m_QRCodesReader->abeQrCodeReaderInit();
        // Une frame est prete
        connect(m_cameraController, SIGNAL(signalFullCvMatImageReady(cv::Mat)),
                this,               SLOT(slotCameraCvMatReceived(cv::Mat)));
        // Passe une frame au scanner
        connect(this,               SIGNAL(signalCvMatForQrdecoder(cv::Mat, bool)),
                m_QRCodesReader,    SLOT(slotAbeQrCodeReaderSetCvMatImage(cv::Mat, bool)));

        // Le scanner a trouvé quelque chose les renseignements sont dans un codeBar
        connect(m_QRCodesReader,    SIGNAL(signalAbeQrCodeReaderNewDecodedDataList(codeBar)),
                this,               SLOT(slotQrCodeDataReceived(codeBar)));
        // Le scanner a une image avec les codeBar encadrés en couleur
        connect(m_QRCodesReader,    SIGNAL(signalAbeQrCodeReaderDecodedCvMatImageReady(cv::Mat)),
                this,               SLOT(slotQrCodeCvMatReceived(cv::Mat)));

        m_scannerThread = new QThread(this);
        m_QRCodesReader->moveToThread(m_scannerThread);
        m_scannerThread->start();
        /* Et hop ! */
    }


    m_cameraController->abeCameraStartCapture();
    ui->teQRCode->clear();
    m_codeBarList.clear();
    }

    void MainWindow::on_btnStop_clicked()
    {
    m_cameraController->abeCameraStopCapture();
    }

    void MainWindow::slotCameraCvMatReceived(cv::Mat frame)
    {
    emit signalCvMatForQrdecoder(frame, false); // Si on passe true, on n'a pas l'image avec les zones des QR-Codes en couleur.
    m_cameraController->slotCameraControllerReadyToGetNewImage(true);
    }

    void MainWindow::slotQrCodeCvMatReceived(cv::Mat frame)
    {

    if(frame.data)
    {
        cv::cvtColor(frame, frame, CV_BGR2RGB);
    }
    QImage image(frame.data, frame.cols, frame.rows, frame.step, QImage::Format_RGB888);
    ui->lbCameraView->setPixmap(QPixmap::fromImage(image).scaled(ui->lbCameraView->size(),Qt::KeepAspectRatio));
    }

    void MainWindow::slotQrCodeDataReceived(codeBar codebar)
    {
    qDebug()<< "Code Bar"<<codebar.type <<codebar.data<< codebar.points;

    m_codeBarList.append(codebar);
    bool modified = true;
    for(int i=0; i<m_codeBarList.size()-1; i++)
    {
        if(m_codeBarList[i].data == codebar.data)
        {
            m_codeBarList.removeLast();
            modified = false;
            break;
        }
    }

    if(modified)
    {
        ui->teQRCode->setText(ui->teQRCode->toPlainText()+"\n"+codebar.type+" : "+codebar.data);
    }

}
