#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QDebug>
#include <QThread>
#include <QLabel>

// opencv
#ifdef WITH_OPENCV2
  #include <opencv2/core/core.hpp>
  #include <opencv2/highgui/highgui.hpp>
  #include <opencv2/imgproc/imgproc.hpp>
#endif

#include "abuleducameracontrollerv1.h"
#include "abuleduqrcodereaderv1.h"
#include "config.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private slots:
    void on_btnStart_clicked();

    void on_btnStop_clicked();

    // On a reçu une frame depuis la caméra
    void slotCameraCvMatReceived(cv::Mat frame);

    // on a reçu une frame depuis le scanner de code bar
    void slotQrCodeCvMatReceived(cv::Mat frame);

    // On a reçu des données depuis le scanner de code bar
    void slotQrCodeDataReceived(codeBar codebar);

    void updateFPS(int fps);
    void updateResolution(int width, int height);

    void on_btnReadQRCodes_clicked();

private:
    Ui::MainWindow *ui;
    AbulEduCameraControllerV1 *m_cameraController;
    AbulEduQrCodeReaderV1   *m_QRCodesReader;
    QList<codeBar> m_codeBarList;
    QThread *m_scannerThread;
    QLabel *lbFPS;
    QLabel *lbRes;

signals:
    // On envoie une image au scanner
    void signalCvMatForQrdecoder(cv::Mat, bool);
};

#endif // MAINWINDOW_H
